@@ -29,14 +29,10 @@ function logger::test_failed() {
 function logger::log() {
   local file="$1"
   local test_name="$2"
-  local start_time="$3"
+  local duration="$3"
   local assertions="$4"
   local status="$5"
 
-  local end_time
-  end_time=$(clock::now)
-  local duration=$((end_time - start_time))
-
   TEST_FILES+=("$file")
   TEST_NAMES+=("$test_name")
   TEST_STATUSES+=("$status")
@@ -57,7 +53,7 @@ function logger::generate_junit_xml() {
   local tests_failed
   tests_failed=$(state::get_tests_failed)
   local time
-  time=$(clock::runtime_in_milliseconds)
+  time=$(clock::total_runtime_in_milliseconds)
 
   {
     echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
@@ -100,7 +96,7 @@ function logger::generate_report_html() {
   local tests_failed
   tests_failed=$(state::get_tests_failed)
   local time
-  time=$(clock::runtime_in_milliseconds)
+  time=$(clock::total_runtime_in_milliseconds)
 
   # Temporary file to store test cases by file
   local temp_file="temp_test_cases.txt"
